import pytest


def pytest_addoption(parser):

    parser.addoption(
        "--release-test",
        action="store_true",
        default=False,
        help="Start release test for all supported calibration notebooks",
    )

    parser.addoption(
        "--detectors",
        default=["all"],
        nargs="+",
    )

    parser.addoption(
        "--calibration",
        type=str.lower,
        choices=["correct", "dark", "all"],
        default="all",
    )

    parser.addoption(
        "--no-numerical-validation",
        action="store_true",
        default=False,
        help="Skips tests for numerical validation for produced h5files.",
    )

    parser.addoption(
        "--validation-only",
        action="store_true",
        default=False,
        help=("Skips running xfel-calibrate CLI and "
              "apply validation test on numerical data only."),
    )

    parser.addoption(
        "--use-slurm",
        action="store_true",
        default=False,
        help=("Run xfel-calibrate CLI without "
              "--no-cluster-job option and use Slurm"),
    )

    parser.addoption(
        "--picked-test",
        type=str,
        default=None,
    )

    parser.addoption(
        "--out-folder",
        type=str,
    )

    parser.addoption(
        "--reference-folder",
        type=str,
    )


@pytest.fixture
def release_test_config(request):
    detectors = request.config.getoption("--detectors")
    calibration = request.config.getoption("--calibration")
    skip_numerical_validation = request.config.getoption(
        "--no-numerical-validation")
    validate_only = request.config.getoption(
        "--validation-only")
    use_slurm = request.config.getoption(
        "--use-slurm")
    picked_test = request.config.getoption("--picked-test")
    reference_folder = request.config.getoption("--reference-folder")
    out_folder = request.config.getoption("--out-folder")
    return (
        detectors, calibration, picked_test,
        skip_numerical_validation, validate_only,
        use_slurm, reference_folder, out_folder,
    )


def pytest_configure(config):
    config.addinivalue_line(
        "markers",
        "manual_run(): marks skips for tests that required to be run manually",
    )


def pytest_runtest_setup(item):
    if list(item.iter_markers(name="manual_run")) and not item.config.getoption(
        "--release-test"
    ):
        pytest.skip("Test initialized manually")
